@@ -15,15 +15,16 @@ from torch.utils.tensorboard import SummaryWriter
 import sbi.inference
 from sbi.inference.posteriors.base_posterior import NeuralPosterior
 from sbi.simulators.simutils import simulate_in_batches
-from sbi.user_input.user_input_checks import prepare_for_sbi
 from sbi.utils import (
     get_log_root,
     handle_invalid_x,
     warn_on_invalid_x,
     warn_on_invalid_x_for_snpec_leakage,
+    warn_if_zscoring_changes_data,
 )
 from sbi.utils.sbiutils import get_simulations_since_round
 from sbi.utils.torchutils import process_device
+from sbi.utils.user_input_checks import prepare_for_sbi
 
 
 def infer(
@@ -85,7 +86,7 @@ class NeuralInference(ABC):
 
     def __init__(
         self,
-        prior,
+        prior: Optional[Any] = None,
         device: str = "cpu",
         logging_level: Union[int, str] = "WARNING",
         summary_writer: Optional[SummaryWriter] = None,
@@ -247,6 +248,8 @@ class NeuralInference(ABC):
 
         # Check for NaNs in simulations.
         is_valid_x, num_nans, num_infs = handle_invalid_x(x, exclude_invalid_x)
+        # Check for problematic z-scoring
+        warn_if_zscoring_changes_data(x)
         if warn_on_invalid:
             warn_on_invalid_x(num_nans, num_infs, exclude_invalid_x)
             warn_on_invalid_x_for_snpec_leakage(
@@ -308,7 +311,9 @@ class NeuralInference(ABC):
 
         method = self.__class__.__name__
         logdir = Path(
-            get_log_root(), method, datetime.now().isoformat().replace(":", "_"),
+            get_log_root(),
+            method,
+            datetime.now().isoformat().replace(":", "_"),
         )
         return SummaryWriter(logdir)
 
@@ -316,8 +321,7 @@ class NeuralInference(ABC):
     def _ensure_list(
         num_simulations_per_round: Union[List[int], int], num_rounds: int
     ) -> List[int]:
-        """Return `num_simulations_per_round` as a list of length `num_rounds`.
-        """
+        """Return `num_simulations_per_round` as a list of length `num_rounds`."""
         try:
             assert len(num_simulations_per_round) == num_rounds, (
                 "Please provide a list with number of simulations per round for each "
@@ -370,7 +374,11 @@ class NeuralInference(ABC):
         assert torch.isfinite(quantity).all(), msg
 
     def _summarize(
-        self, round_: int, x_o: Union[Tensor, None], theta_bank: Tensor, x_bank: Tensor,
+        self,
+        round_: int,
+        x_o: Union[Tensor, None],
+        theta_bank: Tensor,
+        x_bank: Tensor,
     ) -> None:
         """Update the summary_writer with statistics for a given round.
 
@@ -385,7 +393,12 @@ class NeuralInference(ABC):
         # Median |x - x0| for most recent round.
         if x_o is not None:
             median_observation_distance = torch.median(
-                torch.sqrt(torch.sum((x_bank - x_o.reshape(1, -1)) ** 2, dim=-1,))
+                torch.sqrt(
+                    torch.sum(
+                        (x_bank - x_o.reshape(1, -1)) ** 2,
+                        dim=-1,
+                    )
+                )
             )
             self._summary["median_observation_distances"].append(
                 median_observation_distance.item()
@@ -468,7 +481,11 @@ def simulate_for_sbi(
     theta = proposal.sample((num_simulations,))
 
     x = simulate_in_batches(
-        simulator, theta, simulation_batch_size, num_workers, show_progress_bar,
+        simulator,
+        theta,
+        simulation_batch_size,
+        num_workers,
+        show_progress_bar,
     )
 
     return theta, x
