@@ -25,13 +25,13 @@ from sbi.utils import (
     validate_theta_and_x,
     x_shape_from_simulation,
 )
-from sbi.utils.sbiutils import mask_sims_from_prior
+from sbi.utils.sbiutils import ImproperEmpirical, mask_sims_from_prior
 
 
 class PosteriorEstimator(NeuralInference, ABC):
     def __init__(
         self,
-        prior,
+        prior: Optional[Any] = None,
         density_estimator: Union[str, Callable] = "maf",
         device: str = "cpu",
         logging_level: Union[int, str] = "WARNING",
@@ -84,7 +84,10 @@ class PosteriorEstimator(NeuralInference, ABC):
         self._summary.update({"rejection_sampling_acceptance_rates": []})  # type:ignore
 
     def append_simulations(
-        self, theta: Tensor, x: Tensor, proposal: Optional[Any] = None,
+        self,
+        theta: Tensor,
+        x: Tensor,
+        proposal: Optional[Any] = None,
     ) -> "PosteriorEstimator":
         r"""
         Store parameters and simulation outputs to use them for later training.
@@ -134,6 +137,20 @@ class PosteriorEstimator(NeuralInference, ABC):
         self._x_roundwise.append(x)
         self._proposal_roundwise.append(proposal)
 
+        if self._prior is None or isinstance(self._prior, ImproperEmpirical):
+            if proposal is not None:
+                raise ValueError(
+                    "You had not passed a prior at initialization, but now you "
+                    "passed a proposal. If you want to run multi-round SNPE, you have "
+                    "to specify a prior (set the `.prior` argument or re-initialize "
+                    "the object with a prior distribution). If the samples you passed "
+                    "to `append_simulations()` were sampled from the prior, you can "
+                    "run single-round inference with "
+                    "`append_simulations(..., proposal=None)`."
+                )
+            theta_prior = self.get_simulations()[0]
+            self._prior = ImproperEmpirical(theta_prior, ones(theta_prior.shape[0]))
+
         return self
 
     def train(
@@ -146,6 +163,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         clip_max_norm: Optional[float] = 5.0,
         calibration_kernel: Optional[Callable] = None,
         exclude_invalid_x: bool = True,
+        resume_training: bool = False,
         discard_prior_samples: bool = False,
         retrain_from_scratch_each_round: bool = False,
         show_train_summary: bool = False,
@@ -168,6 +186,10 @@ class PosteriorEstimator(NeuralInference, ABC):
                 simulations `x`. See Lueckmann, Gonçalves et al., NeurIPS 2017.
             exclude_invalid_x: Whether to exclude simulation outputs `x=NaN` or `x=±∞`
                 during training. Expect errors, silent or explicit, when `False`.
+            resume_training: Can be used in case training time is limited, e.g. on a
+                cluster. If `True`, the split between train and validation set, the
+                optimizer, the number of epochs, and the best validation log-prob will
+                be restored from the last time `.train()` was called.
             discard_prior_samples: Whether to discard samples simulated in round 1, i.e.
                 from the prior. Training may be sped up by ignoring such less targeted
                 samples.
@@ -189,16 +211,6 @@ class PosteriorEstimator(NeuralInference, ABC):
         # Load data from most recent round.
         theta, x, _ = self.get_simulations(self._round, exclude_invalid_x, False)
 
-        # First round or if retraining from scratch:
-        # Call the `self._build_neural_net` with the rounds' thetas and xs as
-        # arguments, which will build the neural network.
-        # This is passed into NeuralPosterior, to create a neural posterior which
-        # can `sample()` and `log_prob()`. The network is accessible via `.net`.
-        if self._neural_net is None or retrain_from_scratch_each_round:
-            self._neural_net = self._build_neural_net(theta, x)
-            test_posterior_net_for_multi_d_x(self._neural_net, theta, x)
-            self._x_shape = x_shape_from_simulation(x)
-
         # Starting index for the training set (1 = discard round-0 samples).
         start_idx = int(discard_prior_samples and self._round > 0)
 
@@ -216,43 +228,68 @@ class PosteriorEstimator(NeuralInference, ABC):
 
         # Select random neural net and validation splits from (theta, x) pairs.
         num_total_examples = len(theta)
-        permuted_indices = torch.randperm(num_total_examples)
         num_training_examples = int((1 - validation_fraction) * num_total_examples)
         num_validation_examples = num_total_examples - num_training_examples
-        train_indices, val_indices = (
-            permuted_indices[:num_training_examples],
-            permuted_indices[num_training_examples:],
-        )
+
+        if not resume_training:
+            permuted_indices = torch.randperm(num_total_examples)
+            self.train_indices, self.val_indices = (
+                permuted_indices[:num_training_examples],
+                permuted_indices[num_training_examples:],
+            )
 
         # Dataset is shared for training and validation loaders.
-        dataset = data.TensorDataset(theta, x, prior_masks,)
+        dataset = data.TensorDataset(
+            theta,
+            x,
+            prior_masks,
+        )
 
         # Create neural net and validation loaders using a subset sampler.
         train_loader = data.DataLoader(
             dataset,
             batch_size=min(training_batch_size, num_training_examples),
             drop_last=True,
-            sampler=SubsetRandomSampler(train_indices),
+            sampler=SubsetRandomSampler(self.train_indices),
         )
         val_loader = data.DataLoader(
             dataset,
             batch_size=min(training_batch_size, num_validation_examples),
             shuffle=False,
             drop_last=True,
-            sampler=SubsetRandomSampler(val_indices),
+            sampler=SubsetRandomSampler(self.val_indices),
         )
 
+        # First round or if retraining from scratch:
+        # Call the `self._build_neural_net` with the rounds' thetas and xs as
+        # arguments, which will build the neural network.
+        # This is passed into NeuralPosterior, to create a neural posterior which
+        # can `sample()` and `log_prob()`. The network is accessible via `.net`.
+        if self._neural_net is None or retrain_from_scratch_each_round:
+            self._neural_net = self._build_neural_net(
+                theta[self.train_indices], x[self.train_indices]
+            )
+            test_posterior_net_for_multi_d_x(self._neural_net, theta, x)
+            self._x_shape = x_shape_from_simulation(x)
+
         # Move entire net to device for training.
         self._neural_net.to(self._device)
-        optimizer = optim.Adam(list(self._neural_net.parameters()), lr=learning_rate,)
 
-        epoch, self._val_log_prob = 0, float("-Inf")
-        while epoch <= max_num_epochs and not self._converged(epoch, stop_after_epochs):
+        if not resume_training:
+            self.optimizer = optim.Adam(
+                list(self._neural_net.parameters()),
+                lr=learning_rate,
+            )
+            self.epoch, self._val_log_prob = 0, float("-Inf")
+
+        while self.epoch <= max_num_epochs and not self._converged(
+            self.epoch, stop_after_epochs
+        ):
 
             # Train for a single epoch.
             self._neural_net.train()
             for batch in train_loader:
-                optimizer.zero_grad()
+                self.optimizer.zero_grad()
                 # Get batches on current device.
                 theta_batch, x_batch, masks_batch = (
                     batch[0].to(self._device),
@@ -262,17 +299,22 @@ class PosteriorEstimator(NeuralInference, ABC):
 
                 batch_loss = torch.mean(
                     self._loss(
-                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel,
+                        theta_batch,
+                        x_batch,
+                        masks_batch,
+                        proposal,
+                        calibration_kernel,
                     )
                 )
                 batch_loss.backward()
                 if clip_max_norm is not None:
                     clip_grad_norm_(
-                        self._neural_net.parameters(), max_norm=clip_max_norm,
+                        self._neural_net.parameters(),
+                        max_norm=clip_max_norm,
                     )
-                optimizer.step()
+                self.optimizer.step()
 
-            epoch += 1
+            self.epoch += 1
 
             # Calculate validation performance.
             self._neural_net.eval()
@@ -286,7 +328,11 @@ class PosteriorEstimator(NeuralInference, ABC):
                     )
                     # Take negative loss here to get validation log_prob.
                     batch_log_prob = -self._loss(
-                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel,
+                        theta_batch,
+                        x_batch,
+                        masks_batch,
+                        proposal,
+                        calibration_kernel,
                     )
                     log_prob_sum += batch_log_prob.sum().item()
 
@@ -294,17 +340,20 @@ class PosteriorEstimator(NeuralInference, ABC):
             # Log validation log prob for every epoch.
             self._summary["validation_log_probs"].append(self._val_log_prob)
 
-            self._maybe_show_progress(self._show_progress_bars, epoch)
+            self._maybe_show_progress(self._show_progress_bars, self.epoch)
 
-        self._report_convergence_at_end(epoch, stop_after_epochs, max_num_epochs)
+        self._report_convergence_at_end(self.epoch, stop_after_epochs, max_num_epochs)
 
         # Update summary.
-        self._summary["epochs"].append(epoch)
+        self._summary["epochs"].append(self.epoch)
         self._summary["best_validation_log_probs"].append(self._best_val_log_prob)
 
         # Update tensorboard and summary dict.
         self._summarize(
-            round_=self._round, x_o=None, theta_bank=theta, x_bank=x,
+            round_=self._round,
+            x_o=None,
+            theta_bank=theta,
+            x_bank=x,
         )
 
         # Update description for progress bar.
@@ -450,7 +499,10 @@ class PosteriorEstimator(NeuralInference, ABC):
                 )
         elif self._round > 0:
             raise ValueError(
-                "You did not specify a proposal (i.e. `proposal=None`). "
-                "However, previously, you had already specified a proposal. "
-                "This scenario is currently not allowed."
+                "A proposal was passed but no prior was passed at initialisation. When "
+                "running multi-round inference, a prior needs to be specified upon "
+                "initialisation. Potential fix: setting the `._prior` attribute or "
+                "re-initialisation. If the samples passed to `append_simulations()` "
+                "were sampled from the prior, single-round inference can be performed "
+                "with `append_simulations(..., proprosal=None)`."
             )
