@@ -18,6 +18,7 @@ from sbi.types import TensorboardSummaryWriter
 from sbi.utils import (
     batched_mixture_mv,
     batched_mixture_vmv,
+    check_if_boxuniform,
     clamp_and_warn,
     del_entries,
     repeat_rows,
@@ -27,7 +28,7 @@ from sbi.utils import (
 class SNPE_C(PosteriorEstimator):
     def __init__(
         self,
-        prior,
+        prior: Optional[Any] = None,
         density_estimator: Union[str, Callable] = "maf",
         device: str = "cpu",
         logging_level: Union[int, str] = "WARNING",
@@ -100,6 +101,7 @@ class SNPE_C(PosteriorEstimator):
         clip_max_norm: Optional[float] = 5.0,
         calibration_kernel: Optional[Callable] = None,
         exclude_invalid_x: bool = True,
+        resume_training: bool = False,
         discard_prior_samples: bool = False,
         use_combined_loss: bool = False,
         retrain_from_scratch_each_round: bool = False,
@@ -124,6 +126,10 @@ class SNPE_C(PosteriorEstimator):
                 simulations `x`. See Lueckmann, Gonçalves et al., NeurIPS 2017.
             exclude_invalid_x: Whether to exclude simulation outputs `x=NaN` or `x=±∞`
                 during training. Expect errors, silent or explicit, when `False`.
+            resume_training: Can be used in case training time is limited, e.g. on a
+                cluster. If `True`, the split between train and validation set, the
+                optimizer, the number of epochs, and the best validation log-prob will
+                be restored from the last time `.train()` was called.
             discard_prior_samples: Whether to discard samples simulated in round 1, i.e.
                 from the prior. Training may be sped up by ignoring such less targeted
                 samples.
@@ -163,7 +169,7 @@ class SNPE_C(PosteriorEstimator):
                     isinstance(proposal.net._distribution, mdn)
                     and isinstance(self._neural_net._distribution, mdn)
                     and (
-                        isinstance(self._prior, utils.BoxUniform)
+                        check_if_boxuniform(self._prior)[0]
                         or isinstance(self._prior, MultivariateNormal)
                     )
                 )
@@ -244,7 +250,8 @@ class SNPE_C(PosteriorEstimator):
 
             if isinstance(self._prior, MultivariateNormal):
                 self._maybe_z_scored_prior = MultivariateNormal(
-                    almost_zero_mean, torch.diag(almost_one_std),
+                    almost_zero_mean,
+                    torch.diag(almost_one_std),
                 )
             else:
                 range_ = torch.sqrt(almost_one_std * 3.0)
@@ -357,7 +364,10 @@ class SNPE_C(PosteriorEstimator):
         return log_prob_proposal_posterior
 
     def _log_prob_proposal_posterior_mog(
-        self, theta: Tensor, x: Tensor, proposal: DirectPosterior,
+        self,
+        theta: Tensor,
+        x: Tensor,
+        proposal: DirectPosterior,
     ) -> Tensor:
         """
         Return log-probability of the proposal posterior for MoG proposal.
@@ -405,11 +415,21 @@ class SNPE_C(PosteriorEstimator):
 
         # Compute the MoG parameters of the proposal posterior.
         logits_pp, m_pp, prec_pp, cov_pp = self._automatic_posterior_transformation(
-            norm_logits_p, m_p, prec_p, norm_logits_d, m_d, prec_d,
+            norm_logits_p,
+            m_p,
+            prec_p,
+            norm_logits_d,
+            m_d,
+            prec_d,
         )
 
         # Compute the log_prob of theta under the product.
-        log_prob_proposal_posterior = _mog_log_prob(theta, logits_pp, m_pp, prec_pp,)
+        log_prob_proposal_posterior = _mog_log_prob(
+            theta,
+            logits_pp,
+            m_pp,
+            prec_pp,
+        )
         self._assert_all_finite(log_prob_proposal_posterior, "proposal posterior eval")
 
         return log_prob_proposal_posterior
@@ -463,7 +483,11 @@ class SNPE_C(PosteriorEstimator):
         )
 
         means_pp = self._means_proposal_posterior(
-            covariances_pp, means_p, precisions_p, means_d, precisions_d,
+            covariances_pp,
+            means_p,
+            precisions_p,
+            means_d,
+            precisions_d,
         )
 
         logits_pp = self._logits_proposal_posterior(
@@ -481,7 +505,9 @@ class SNPE_C(PosteriorEstimator):
         return logits_pp, means_pp, precisions_pp, covariances_pp
 
     def _precisions_proposal_posterior(
-        self, precisions_p: Tensor, precisions_d: Tensor,
+        self,
+        precisions_p: Tensor,
+        precisions_d: Tensor,
     ):
         """
         Return the precisions and covariances of the proposal posterior.
@@ -629,7 +655,10 @@ class SNPE_C(PosteriorEstimator):
 
 
 def _mog_log_prob(
-    theta: Tensor, logits_pp: Tensor, means_pp: Tensor, precisions_pp: Tensor,
+    theta: Tensor,
+    logits_pp: Tensor,
+    means_pp: Tensor,
+    precisions_pp: Tensor,
 ) -> Tensor:
     r"""
     Returns the log-probability of parameter sets $\theta$ under a mixture of Gaussians.
