@@ -62,6 +62,7 @@ class SNRE_B(RatioEstimator):
         max_num_epochs: Optional[int] = None,
         clip_max_norm: Optional[float] = 5.0,
         exclude_invalid_x: bool = True,
+        resume_training: bool = False,
         discard_prior_samples: bool = False,
         retrain_from_scratch_each_round: bool = False,
         show_train_summary: bool = False,
@@ -83,6 +84,10 @@ class SNRE_B(RatioEstimator):
                 prevent exploding gradients. Use None for no clipping.
             exclude_invalid_x: Whether to exclude simulation outputs `x=NaN` or `x=±∞`
                 during training. Expect errors, silent or explicit, when `False`.
+            resume_training: Can be used in case training time is limited, e.g. on a
+                cluster. If `True`, the split between train and validation set, the
+                optimizer, the number of epochs, and the best validation log-prob will
+                be restored from the last time `.train()` was called.
             discard_prior_samples: Whether to discard samples simulated in round 1, i.e.
                 from the prior. Training may be sped up by ignoring such less targeted
                 samples.
