@@ -82,7 +82,10 @@ class RatioEstimator(NeuralInference, ABC):
         self._summary.update({"mcmc_times": []})  # type: ignore
 
     def append_simulations(
-        self, theta: Tensor, x: Tensor, from_round: int = 0,
+        self,
+        theta: Tensor,
+        x: Tensor,
+        from_round: int = 0,
     ) -> "RatioEstimator":
         r"""
         Store parameters and simulation outputs to use them for later training.
@@ -123,6 +126,7 @@ class RatioEstimator(NeuralInference, ABC):
         max_num_epochs: Optional[int] = None,
         clip_max_norm: Optional[float] = 5.0,
         exclude_invalid_x: bool = True,
+        resume_training: bool = False,
         discard_prior_samples: bool = False,
         retrain_from_scratch_each_round: bool = False,
         show_train_summary: bool = False,
@@ -134,6 +138,10 @@ class RatioEstimator(NeuralInference, ABC):
             num_atoms: Number of atoms to use for classification.
             exclude_invalid_x: Whether to exclude simulation outputs `x=NaN` or `x=±∞`
                 during training. Expect errors, silent or explicit, when `False`.
+            resume_training: Can be used in case training time is limited, e.g. on a
+                cluster. If `True`, the split between train and validation set, the
+                optimizer, the number of epochs, and the best validation log-prob will
+                be restored from the last time `.train()` was called.
             discard_prior_samples: Whether to discard samples simulated in round 1, i.e.
                 from the prior. Training may be sped up by ignoring such less targeted
                 samples.
@@ -150,19 +158,6 @@ class RatioEstimator(NeuralInference, ABC):
         self._round = max(self._data_round_index)
         theta, x, _ = self.get_simulations(self._round, exclude_invalid_x, False)
 
-        # First round or if retraining from scratch:
-        # Call the `self._build_neural_net` with the rounds' thetas and xs as
-        # arguments, which will build the neural network
-        # This is passed into NeuralPosterior, to create a neural posterior which
-        # can `sample()` and `log_prob()`. The network is accessible via `.net`.
-        if self._neural_net is None or retrain_from_scratch_each_round:
-            self._neural_net = self._build_neural_net(theta, x)
-            self._x_shape = x_shape_from_simulation(x)
-            assert len(self._x_shape) < 3, (
-                "For now, SNRE cannot handle multi-dimensional simulator output, see "
-                "issue #360."
-            )
-
         # Starting index for the training set (1 = discard round-0 samples).
         start_idx = int(discard_prior_samples and self._round > 0)
         theta, x, _ = self.get_simulations(start_idx, exclude_invalid_x)
@@ -171,13 +166,15 @@ class RatioEstimator(NeuralInference, ABC):
         num_examples = len(theta)
 
         # Select random train and validation splits from (theta, x) pairs.
-        permuted_indices = torch.randperm(num_examples)
         num_training_examples = int((1 - validation_fraction) * num_examples)
         num_validation_examples = num_examples - num_training_examples
-        train_indices, val_indices = (
-            permuted_indices[:num_training_examples],
-            permuted_indices[num_training_examples:],
-        )
+
+        if not resume_training:
+            permuted_indices = torch.randperm(num_examples)
+            self.train_indices, self.val_indices = (
+                permuted_indices[:num_training_examples],
+                permuted_indices[num_training_examples:],
+            )
 
         clipped_batch_size = min(training_batch_size, num_validation_examples)
 
@@ -193,27 +190,44 @@ class RatioEstimator(NeuralInference, ABC):
             dataset,
             batch_size=clipped_batch_size,
             drop_last=True,
-            sampler=SubsetRandomSampler(train_indices),
+            sampler=SubsetRandomSampler(self.train_indices),
         )
         val_loader = data.DataLoader(
             dataset,
             batch_size=clipped_batch_size,
             shuffle=False,
             drop_last=False,
-            sampler=SubsetRandomSampler(val_indices),
+            sampler=SubsetRandomSampler(self.val_indices),
         )
 
+        # First round or if retraining from scratch:
+        # Call the `self._build_neural_net` with the rounds' thetas and xs as
+        # arguments, which will build the neural network
+        # This is passed into NeuralPosterior, to create a neural posterior which
+        # can `sample()` and `log_prob()`. The network is accessible via `.net`.
+        if self._neural_net is None or retrain_from_scratch_each_round:
+            self._neural_net = self._build_neural_net(
+                theta[self.train_indices], x[self.train_indices]
+            )
+            self._x_shape = x_shape_from_simulation(x)
+
         self._neural_net.to(self._device)
-        optimizer = optim.Adam(list(self._neural_net.parameters()), lr=learning_rate,)
 
-        epoch, self._val_log_prob = 0, float("-Inf")
+        if not resume_training:
+            self.optimizer = optim.Adam(
+                list(self._neural_net.parameters()),
+                lr=learning_rate,
+            )
+            self.epoch, self._val_log_prob = 0, float("-Inf")
 
-        while epoch <= max_num_epochs and not self._converged(epoch, stop_after_epochs):
+        while self.epoch <= max_num_epochs and not self._converged(
+            self.epoch, stop_after_epochs
+        ):
 
             # Train for a single epoch.
             self._neural_net.train()
             for batch in train_loader:
-                optimizer.zero_grad()
+                self.optimizer.zero_grad()
                 theta_batch, x_batch = (
                     batch[0].to(self._device),
                     batch[1].to(self._device),
@@ -222,11 +236,12 @@ class RatioEstimator(NeuralInference, ABC):
                 loss.backward()
                 if clip_max_norm is not None:
                     clip_grad_norm_(
-                        self._neural_net.parameters(), max_norm=clip_max_norm,
+                        self._neural_net.parameters(),
+                        max_norm=clip_max_norm,
                     )
-                optimizer.step()
+                self.optimizer.step()
 
-            epoch += 1
+            self.epoch += 1
 
             # Calculate validation performance.
             self._neural_net.eval()
@@ -243,17 +258,20 @@ class RatioEstimator(NeuralInference, ABC):
                 # Log validation log prob for every epoch.
                 self._summary["validation_log_probs"].append(self._val_log_prob)
 
-            self._maybe_show_progress(self._show_progress_bars, epoch)
+            self._maybe_show_progress(self._show_progress_bars, self.epoch)
 
-        self._report_convergence_at_end(epoch, stop_after_epochs, max_num_epochs)
+        self._report_convergence_at_end(self.epoch, stop_after_epochs, max_num_epochs)
 
         # Update summary.
-        self._summary["epochs"].append(epoch)
+        self._summary["epochs"].append(self.epoch)
         self._summary["best_validation_log_probs"].append(self._best_val_log_prob)
 
         # Update TensorBoard and summary dict.
         self._summarize(
-            round_=self._round, x_o=None, theta_bank=theta, x_bank=x,
+            round_=self._round,
+            x_o=None,
+            theta_bank=theta,
+            x_bank=x,
         )
 
         # Update description for progress bar.
@@ -346,9 +364,7 @@ class RatioEstimator(NeuralInference, ABC):
             batch_size * num_atoms, -1
         )
 
-        theta_and_x = torch.cat((atomic_theta, repeated_x), dim=1)
-
-        return self._neural_net(theta_and_x)
+        return self._neural_net([atomic_theta, repeated_x])
 
     @abstractmethod
     def _loss(self, theta: Tensor, x: Tensor, num_atoms: int) -> Tensor:
