@@ -1,4 +1,4 @@
 from sbi.mcmc.slice_numpy import SliceSampler
 from sbi.mcmc.slice_numpy_vectorized import SliceSamplerVectorized
 from sbi.mcmc.slice import Slice
-from sbi.mcmc.init_strategy import prior_init, sir
+from sbi.mcmc.init_strategy import IterateParameters, prior_init, sir
