@@ -2,19 +2,8 @@
 # under the Affero General Public License v3, see <https://www.gnu.org/licenses/>.
 
 import collections
-import inspect
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    TypeVar,
-    Union,
-    cast,
-)
+from typing import Any, List, Optional, Tuple, Union
+from warnings import warn
 
 import matplotlib as mpl
 import numpy as np
@@ -150,9 +139,11 @@ def pairplot(
     subset: List[int] = None,
     upper: Optional[str] = "hist",
     diag: Optional[str] = "hist",
-    fig_size: Tuple = (10, 10),
+    figsize: Tuple = (10, 10),
     labels: Optional[List[str]] = None,
+    ticks: Union[List, torch.Tensor] = [],
     points_colors: List[str] = plt.rcParams["axes.prop_cycle"].by_key()["color"],
+    warn_about_deprecation: bool = True,
     **kwargs
 ):
     """
@@ -172,15 +163,25 @@ def pairplot(
             if they exist, the 4th, 5th and so on).
         upper: Plotting style for upper diagonal, {hist, scatter, contour, cond, None}.
         diag: Plotting style for diagonal, {hist, cond, None}.
-        fig_size: Size of the entire figure.
+        figsize: Size of the entire figure.
         labels: List of strings specifying the names of the parameters.
+        ticks: Position of the ticks.
         points_colors: Colors of the `points`.
+        warn_about_deprecation: With sbi v0.15.0, we depracated the import of this
+            function from `sbi.utils`. Instead, it should be imported from
+            `sbi.analysis`.
         **kwargs: Additional arguments to adjust the plot, see the source code in
             `_get_default_opts()` in `sbi.utils.plot` for more details.
 
     Returns: figure and axis of posterior distribution plot
     """
 
+    if warn_about_deprecation:
+        warn(
+            "Importing `pairplot` from `sbi.utils` is deprecated since sbi "
+            "v0.15.0. Instead, use `from sbi.analysis import pairplot`."
+        )
+
     # TODO: add color map support
     # TODO: automatically determine good bin sizes for histograms
     # TODO: add legend (if legend is True)
@@ -188,9 +189,18 @@ def pairplot(
     opts = _get_default_opts()
     # update the defaults dictionary by the current values of the variables (passed by
     # the user)
+
     opts = _update(opts, locals())
     opts = _update(opts, kwargs)
 
+    if "fig_size" in opts.keys():
+        warn(
+            "You passed an argument `fig_size`. Since sbi v0.15.0, the argument "
+            "should be called `figsize`. In future versions, `fig_size` will no longer "
+            "be supported."
+        )
+        opts["figsize"] = opts["fig_size"]
+
     # Prepare samples
     if type(samples) != list:
         samples = ensure_numpy(samples)
@@ -245,7 +255,11 @@ def pairplot(
                         limits[row, 0], limits[row, 1], opts["kde_diag"]["bins"]
                     )
                     ys = density(xs)
-                    h = plt.plot(xs, ys, color=opts["samples_colors"][n],)
+                    h = plt.plot(
+                        xs,
+                        ys,
+                        color=opts["samples_colors"][n],
+                    )
                 else:
                     pass
 
@@ -265,7 +279,12 @@ def pairplot(
                     h = plt.imshow(
                         hist.T,
                         origin="lower",
-                        extent=[xedges[0], xedges[-1], yedges[0], yedges[-1],],
+                        extent=[
+                            xedges[0],
+                            xedges[-1],
+                            yedges[0],
+                            yedges[-1],
+                        ],
                         aspect="auto",
                     )
 
@@ -276,14 +295,19 @@ def pairplot(
                     "contourf",
                 ]:
                     density = gaussian_kde(
-                        v[:, [col, row]].T, bw_method=opts["kde_offdiag"]["bw_method"],
+                        v[:, [col, row]].T,
+                        bw_method=opts["kde_offdiag"]["bw_method"],
                     )
                     X, Y = np.meshgrid(
                         np.linspace(
-                            limits[col][0], limits[col][1], opts["kde_offdiag"]["bins"],
+                            limits[col][0],
+                            limits[col][1],
+                            opts["kde_offdiag"]["bins"],
                         ),
                         np.linspace(
-                            limits[row][0], limits[row][1], opts["kde_offdiag"]["bins"],
+                            limits[row][0],
+                            limits[row][1],
+                            opts["kde_offdiag"]["bins"],
                         ),
                     )
                     positions = np.vstack([X.ravel(), Y.ravel()])
@@ -351,9 +375,11 @@ def conditional_pairplot(
     ] = None,
     subset: List[int] = None,
     resolution: int = 50,
-    fig_size: Tuple = (10, 10),
+    figsize: Tuple = (10, 10),
     labels: Optional[List[str]] = None,
+    ticks: Union[List, torch.Tensor] = [],
     points_colors: List[str] = plt.rcParams["axes.prop_cycle"].by_key()["color"],
+    warn_about_deprecation: bool = True,
     **kwargs
 ):
     r"""
@@ -382,15 +408,25 @@ def conditional_pairplot(
             plot only the 1st and 3rd dimension but will discard the 0th and 2nd (and,
             if they exist, the 4th, 5th and so on)
         resolution: Resolution of the grid at which we evaluate the `pdf`.
-        fig_size: Size of the entire figure.
+        figsize: Size of the entire figure.
         labels: List of strings specifying the names of the parameters.
+        ticks: Position of the ticks.
         points_colors: Colors of the `points`.
+        warn_about_deprecation: With sbi v0.15.0, we depracated the import of this
+            function from `sbi.utils`. Instead, it should be imported from
+            `sbi.analysis`.
         **kwargs: Additional arguments to adjust the plot, see the source code in
             `_get_default_opts()` in `sbi.utils.plot` for more details.
 
     Returns: figure and axis of posterior distribution plot
     """
 
+    if warn_about_deprecation:
+        warn(
+            "Importing `conditional_pairplot` from `sbi.utils` is deprecated since sbi "
+            "v0.15.0. Instead, use `from sbi.analysis import conditional_pairplot`."
+        )
+
     # Setting these is required because _pairplot_scaffold will check if opts['diag'] is
     # `None`. This would break if opts has no key 'diag'. Same for 'upper'.
     diag = "cond"
@@ -402,6 +438,14 @@ def conditional_pairplot(
     opts = _update(opts, locals())
     opts = _update(opts, kwargs)
 
+    if "fig_size" in opts.keys():
+        warn(
+            "You passed an argument `fig_size`. Since sbi v0.15.0, the argument "
+            "should be called `figsize`. In future versions, `fig_size` will no longer "
+            "be supported."
+        )
+        opts["figsize"] = opts["fig_size"]
+
     # Dimensions
     dim = condition.shape[-1]
 
@@ -431,7 +475,11 @@ def conditional_pairplot(
             eps_margins2=eps_margins[row],
         ).numpy()
         h = plt.plot(
-            np.linspace(limits[row, 0], limits[row, 1], resolution,),
+            np.linspace(
+                limits[row, 0],
+                limits[row, 1],
+                resolution,
+            ),
             p_vector,
             c=opts["samples_colors"][0],
         )
@@ -450,7 +498,12 @@ def conditional_pairplot(
         h = plt.imshow(
             p_image.T,
             origin="lower",
-            extent=[limits[col, 0], limits[col, 1], limits[row, 0], limits[row, 1],],
+            extent=[
+                limits[col, 0],
+                limits[col, 1],
+                limits[row, 0],
+                limits[row, 1],
+            ],
             aspect="auto",
         )
 
@@ -472,7 +525,7 @@ def _pairplot_scaffold(diag_func, upper_func, dim, limits, points, opts):
         limits: Limits for each parameter.
         points: Additional points to be scatter-plotted.
         opts: Dictionary built by the functions that call `pairplot_scaffold`. Must
-            contain at least `labels`, `ticks`, `subset`, `fig_size`, `subplots`,
+            contain at least `labels`, `subset`, `figsize`, `subplots`,
             `fig_subplots_adjust`, `title`, `title_format`, ..
 
     Returns: figure and axis
@@ -518,7 +571,7 @@ def _pairplot_scaffold(diag_func, upper_func, dim, limits, points, opts):
             raise NotImplementedError
         rows = cols = len(subset)
 
-    fig, axes = plt.subplots(rows, cols, figsize=opts["fig_size"], **opts["subplots"])
+    fig, axes = plt.subplots(rows, cols, figsize=opts["figsize"], **opts["subplots"])
     # Cast to ndarray in case of 1D subplots.
     axes = np.array(axes).reshape(rows, cols)
 
@@ -627,7 +680,9 @@ def _pairplot_scaffold(diag_func, upper_func, dim, limits, points, opts):
             # Off-diagonals
             else:
                 upper_func(
-                    row=row, col=col, limits=limits,
+                    row=row,
+                    col=col,
+                    limits=limits,
                 )
 
                 if len(points) > 0:
@@ -673,7 +728,6 @@ def _get_default_opts():
         # colors
         "samples_colors": plt.rcParams["axes.prop_cycle"].by_key()["color"],
         # ticks
-        "ticks": [],
         "tickformatter": mpl.ticker.FormatStrFormatter("%g"),
         "tick_labels": None,
         # options for hist
@@ -689,16 +743,27 @@ def _get_default_opts():
         # options for contour
         "contour_offdiag": {"levels": [0.68], "percentile": True},
         # options for scatter
-        "scatter_offdiag": {"alpha": 0.5, "edgecolor": "none", "rasterized": False,},
+        "scatter_offdiag": {
+            "alpha": 0.5,
+            "edgecolor": "none",
+            "rasterized": False,
+        },
         # options for plot
         "plot_offdiag": {},
         # formatting points (scale, markers)
         "points_diag": {},
-        "points_offdiag": {"marker": ".", "markersize": 20,},
+        "points_offdiag": {
+            "marker": ".",
+            "markersize": 20,
+        },
         # other options
         "fig_bg_colors": {"upper": None, "diag": None, "lower": None},
-        "fig_subplots_adjust": {"top": 0.9,},
+        "fig_subplots_adjust": {
+            "top": 0.9,
+        },
         "subplots": {},
-        "despine": {"offset": 5,},
+        "despine": {
+            "offset": 5,
+        },
         "title_format": {"fontsize": 16},
     }
