@@ -1,6 +1,4 @@
-from typing import Callable, Optional, Union, Dict, Any, Tuple, Union, cast, List, Sequence, TypeVar
-
-from pyro import distributions as dist
+from typing import Callable, Any
 from pyro import poutine as poutine
 from torch.distributions import biject_to
 
