@@ -2,15 +2,49 @@
 # under the Affero General Public License v3, see <https://www.gnu.org/licenses/>.
 
 import logging
-from typing import Any, Dict, List, Sequence, Tuple, Union
+import warnings
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import torch
 from pyknos.nflows import transforms
+from pyro.distributions import Empirical
 from torch import Tensor, as_tensor
 from torch import nn as nn
 from torch import ones, zeros
 from tqdm.auto import tqdm
 
+from sbi.utils.torchutils import BoxUniform, atleast_2d
+
+
+def warn_if_zscoring_changes_data(x: Tensor, duplicate_tolerance: float = 0.1) -> None:
+    """Raise warning if z-scoring would create duplicate data points.
+
+    Args:
+        x: Simulation outputs.
+        duplicate_tolerance: Tolerated proportion of duplicates after z-scoring.
+    """
+
+    # Count unique xs.
+    num_unique = torch.unique(x, dim=0).numel()
+
+    # z-score.
+    zx = (x - x.mean(0)) / x.std(0)
+
+    # Count again and warn on too many new duplicates.
+    num_unique_z = torch.unique(zx, dim=0).numel()
+
+    if num_unique_z < num_unique * (1 - duplicate_tolerance):
+        warnings.warn(
+            """Z-scoring these simulation outputs resulted in {num_unique_z} unique
+            datapoints. Before z-scoring, it had been {num_unique}. This can occur due
+            to numerical inaccuracies when the data covers a large range of values.
+            Consider either setting `z_score_x=False` (but beware that this can be
+            problematic for training the NN) or exclude outliers from your dataset.
+            Note: if you have already set `z_score_x=False`, this warning will still be
+            displayed, but you can ignore it.""",
+            UserWarning,
+        )
+
 
 def x_shape_from_simulation(batch_x: Tensor) -> torch.Size:
     ndims = batch_x.ndim
@@ -371,3 +405,92 @@ def batched_mixture_mv(matrix: Tensor, vector: Tensor) -> Tensor:
         Product (matrix * vector) of shape (batch_dim, num_components, parameter_dim).
     """
     return torch.einsum("bcij,bcj -> bci", matrix, vector)
+
+
+def expit(theta_t: Tensor, lower_bound: Tensor, upper_bound: Tensor) -> Tensor:
+    """
+    Return the expit() of an input.
+
+    The `expit` transforms an unbounded input to the interval
+    `[lower_bound, upper_bound]`.
+
+    Args:
+        theta_t: Input to be transformed.
+        lower_bound: Lower bound of the transformation.
+        upper_bound: Upper bound of the transformation.
+
+    Returns: theta that is bounded between `lower_bound` and `upper_bound`.
+    """
+    range_ = upper_bound - lower_bound
+    return range_ / (1 + torch.exp(-theta_t)) + lower_bound
+
+
+def logit(theta: Tensor, lower_bound: Tensor, upper_bound: Tensor) -> Tensor:
+    """
+    Return the logit() of an input.
+
+    The `logit` maps the interval `[lower_bound, upper_bound]` to an unbounded space.
+
+    Args:
+        theta: Input to be transformed.
+        lower_bound: Lower bound of the transformation.
+        upper_bound: Upper bound of the transformation.
+
+    Returns: theta_t that is unbounded.
+    """
+    range_ = upper_bound - lower_bound
+    theta_01 = (theta - lower_bound) / range_
+
+    return torch.log(theta_01 / (1 - theta_01))
+
+
+def check_if_boxuniform(dist) -> Tuple[bool, Optional[BoxUniform]]:
+    """Returns whether the `dist` is `BoxUniform` as well as the `BoxUniform` itself.
+
+    When the user called `prepare_for_sbi`, the distribution will in fact be a
+    `PytorchReturnTypeWrapper`. Thus, we need additional checks.
+
+    Args:
+        dist: Distribution to be checked.
+
+    Returns:
+        Whether the `dist` is `BoxUniform` and the `BoxUniform` itself.
+    """
+    if isinstance(dist, BoxUniform):
+        is_boxuniform = True
+        box_dist = dist
+    elif hasattr(dist, "prior") and isinstance(dist.prior, BoxUniform):
+        is_boxuniform = True
+        box_dist = dist.prior
+    else:
+        is_boxuniform = False
+        box_dist = None
+
+    return is_boxuniform, box_dist
+
+
+class ImproperEmpirical(Empirical):
+    """
+    Wrapper around pyro's `Emprirical` distribution that returns constant `log_prob()`.
+
+    This class is used in SNPE when no prior is passed. Having a constant
+    log-probability will lead to no samples being rejected during rejection-sampling.
+
+    The default behavior of `pyro.distributions.Empirical` is that it returns `-inf`
+    for any value that does not **exactly** match one of the samples passed at
+    initialization. Thus, all posterior samples would be rejected for not fitting this
+    criterion.
+    """
+
+    def log_prob(self, value: Tensor) -> Tensor:
+        """
+        Return ones as a constant log-prob for each input.
+
+        Args:
+            value: The parameters at which to evaluate the log-probability.
+
+        Returns:
+            Tensor of as many ones as there were parameter sets.
+        """
+        value = atleast_2d(value)
+        return zeros(value.shape[0])
