@@ -62,6 +62,7 @@ def test_c2st_snl_on_linearGaussian_different_dims(set_seed):
 
     x_o = ones(1, x_dim)
     num_samples = 1000
+    num_simulations = 5000
 
     # likelihood_mean will be likelihood_shift+theta
     likelihood_shift = -1.0 * ones(x_dim)
@@ -87,7 +88,9 @@ def test_c2st_snl_on_linearGaussian_different_dims(set_seed):
     simulator, prior = prepare_for_sbi(simulator, prior)
     inference = SNL(prior, show_progress_bars=False,)
 
-    theta, x = simulate_for_sbi(simulator, prior, 5000, simulation_batch_size=50)
+    theta, x = simulate_for_sbi(
+        simulator, prior, num_simulations, simulation_batch_size=50
+    )
     _ = inference.append_simulations(theta, x).train()
     posterior = inference.build_posterior()
     samples = posterior.sample((num_samples,), x=x_o, mcmc_parameters={"thin": 3})
@@ -143,6 +146,8 @@ def test_c2st_snl_on_linearGaussian(num_dim: int, prior_str: str, set_seed):
     # Check performance based on c2st accuracy.
     check_c2st(samples, target_samples, alg=f"snle_a-{prior_str}-prior")
 
+    map_ = posterior.map(num_init_samples=1_000, init_method="prior")
+
     # TODO: we do not have a test for SNL log_prob(). This is because the output
     # TODO: density is not normalized, so KLd does not make sense.
     if prior_str == "uniform":
@@ -152,6 +157,10 @@ def test_c2st_snl_on_linearGaussian(num_dim: int, prior_str: str, set_seed):
             posterior_prob == 0.0
         ), "The posterior probability outside of the prior support is not zero"
 
+        assert ((map_ - ones(num_dim)) ** 2).sum() < 0.5
+    else:
+        assert ((map_ - gt_posterior.mean) ** 2).sum() < 0.5
+
 
 @pytest.mark.slow
 def test_c2st_multi_round_snl_on_linearGaussian(set_seed):
