@@ -10,6 +10,7 @@ from scipy.stats import gaussian_kde
 from torch import eye, ones, zeros
 from torch.distributions import MultivariateNormal
 
+from sbi import analysis as analysis
 from sbi import utils as utils
 from sbi.inference import SNPE_B, SNPE_C, prepare_for_sbi, simulate_for_sbi
 from sbi.simulators.linear_gaussian import (
@@ -27,10 +28,17 @@ from tests.test_utils import (
 
 
 @pytest.mark.parametrize(
-    "num_dim, prior_str", ((2, "gaussian"), (2, "uniform"), (1, "gaussian"),),
+    "num_dim, prior_str",
+    (
+        (2, "gaussian"),
+        (2, "uniform"),
+        (1, "gaussian"),
+    ),
 )
 def test_c2st_snpe_on_linearGaussian(
-    num_dim: int, prior_str: str, set_seed,
+    num_dim: int,
+    prior_str: str,
+    set_seed,
 ):
     """Test whether SNPE C infers well a simple example with available ground truth.
 
@@ -63,7 +71,10 @@ def test_c2st_snpe_on_linearGaussian(
         return linear_gaussian(theta, likelihood_shift, likelihood_cov)
 
     simulator, prior = prepare_for_sbi(simulator, prior)
-    inference = SNPE_C(prior, show_progress_bars=False,)
+    inference = SNPE_C(
+        prior,
+        show_progress_bars=False,
+    )
 
     theta, x = simulate_for_sbi(simulator, prior, 2000, simulation_batch_size=1000)
     _ = inference.append_simulations(theta, x).train(training_batch_size=100)
@@ -73,6 +84,8 @@ def test_c2st_snpe_on_linearGaussian(
     # Compute the c2st and assert it is near chance level of 0.5.
     check_c2st(samples, target_samples, alg="snpe_c")
 
+    map_ = posterior.map(num_init_samples=1_000)
+
     # Checks for log_prob()
     if prior_str == "gaussian":
         # For the Gaussian prior, we compute the KLd between ground truth and posterior.
@@ -86,6 +99,8 @@ def test_c2st_snpe_on_linearGaussian(
             dkl < max_dkl
         ), f"D-KL={dkl} is more than 2 stds above the average performance."
 
+        assert ((map_ - gt_posterior.mean) ** 2).sum() < 0.5
+
     elif prior_str == "uniform":
         # Check whether the returned probability outside of the support is zero.
         posterior_prob = get_prob_outside_uniform_prior(posterior, num_dim)
@@ -111,12 +126,15 @@ def test_c2st_snpe_on_linearGaussian(
             < acceptance_prob * 1.01
         ), "Normalizing the posterior density using the acceptance probability failed."
 
+        assert ((map_ - ones(num_dim)) ** 2).sum() < 0.5
+
 
 def test_c2st_snpe_on_linearGaussian_different_dims(set_seed):
     """Test whether SNPE B/C infer well a simple example with available ground truth.
 
     This example has different number of parameters theta than number of x. Also
-    this implicitly tests simulation_batch_size=1.
+    this implicitly tests simulation_batch_size=1. It also impleictly tests whether the
+    prior can be `None` and whether we can stop and resume training.
 
     Args:
         set_seed: fixture for manual seeding
@@ -152,10 +170,16 @@ def test_c2st_snpe_on_linearGaussian_different_dims(set_seed):
         )
 
     simulator, prior = prepare_for_sbi(simulator, prior)
-    inference = SNPE_C(prior, density_estimator="maf", show_progress_bars=False,)
+    inference = SNPE_C(
+        prior=None,  # Test whether prior can be `None`.
+        density_estimator="maf",
+        show_progress_bars=False,
+    )
 
     theta, x = simulate_for_sbi(simulator, prior, 2000, simulation_batch_size=1)  # type: ignore
-    _ = inference.append_simulations(theta, x).train()
+    inference = inference.append_simulations(theta, x)
+    _ = inference.train(max_num_epochs=10)  # Test whether we can stop and resume.
+    _ = inference.train(resume_training=True)
     posterior = inference.build_posterior()
     samples = posterior.sample((num_samples,), x=x_o)
 
@@ -171,7 +195,8 @@ def test_c2st_snpe_on_linearGaussian_different_dims(set_seed):
         pytest.param(
             "snpe_b",
             marks=pytest.mark.xfail(
-                raises=NotImplementedError, reason="""SNPE-B not implemented""",
+                raises=NotImplementedError,
+                reason="""SNPE-B not implemented""",
             ),
         ),
         "snpe_c",
@@ -341,7 +366,11 @@ def test_sample_conditional(set_seed):
     net = utils.posterior_nn("maf", hidden_features=20)
 
     simulator, prior = prepare_for_sbi(simulator, prior)
-    inference = SNPE_C(prior, density_estimator=net, show_progress_bars=False,)
+    inference = SNPE_C(
+        prior,
+        density_estimator=net,
+        show_progress_bars=False,
+    )
 
     # We need a pretty big dataset to properly model the bimodality.
     theta, x = simulate_for_sbi(simulator, prior, 10000)
@@ -354,7 +383,7 @@ def test_sample_conditional(set_seed):
     cond_samples = posterior.sample_conditional(
         (500,), condition=samples[0], dims_to_sample=[dim_to_sample_1, dim_to_sample_2]
     )
-    _ = utils.pairplot(
+    _ = analysis.pairplot(
         cond_samples,
         limits=[[-2, 2], [-2, 2], [-2, 2]],
         fig_size=(2, 2),
@@ -367,14 +396,22 @@ def test_sample_conditional(set_seed):
     density = gaussian_kde(cond_samples.numpy().T, bw_method="scott")
 
     X, Y = np.meshgrid(
-        np.linspace(limits[0][0], limits[0][1], 50,),
-        np.linspace(limits[1][0], limits[1][1], 50,),
+        np.linspace(
+            limits[0][0],
+            limits[0][1],
+            50,
+        ),
+        np.linspace(
+            limits[1][0],
+            limits[1][1],
+            50,
+        ),
     )
     positions = np.vstack([X.ravel(), Y.ravel()])
     sample_kde_grid = np.reshape(density(positions).T, X.shape)
 
     # Evaluate the conditional with eval_conditional_density.
-    eval_grid = utils.eval_conditional_density(
+    eval_grid = analysis.eval_conditional_density(
         posterior,
         condition=samples[0],
         dim1=dim_to_sample_1,
@@ -409,7 +446,10 @@ def example_posterior():
         return linear_gaussian(theta, likelihood_shift, likelihood_cov)
 
     simulator, prior = prepare_for_sbi(simulator, prior)
-    inference = SNPE_C(prior, show_progress_bars=False,)
+    inference = SNPE_C(
+        prior,
+        show_progress_bars=False,
+    )
     theta, x = simulate_for_sbi(
         simulator, prior, 1000, simulation_batch_size=10, num_workers=6
     )
