@@ -25,20 +25,21 @@ from torch.distributions import Beta, Distribution, Gamma, MultivariateNormal, U
 
 from sbi.inference import SNPE_C, simulate_for_sbi
 from sbi.simulators.linear_gaussian import diagonal_linear_gaussian
-from sbi.user_input.user_input_checks import (
+from sbi.utils.get_nn_models import posterior_nn
+from sbi.utils.torchutils import BoxUniform
+from sbi.utils.user_input_checks import (
     prepare_for_sbi,
     process_prior,
     process_simulator,
     process_x,
+    validate_theta_and_x
 )
-from sbi.user_input.user_input_checks_utils import (
+from sbi.utils.user_input_checks_utils import (
     CustomPytorchWrapper,
     MultipleIndependent,
     PytorchReturnTypeWrapper,
     ScipyPytorchWrapper,
 )
-from sbi.utils.get_nn_models import posterior_nn
-from sbi.utils.torchutils import BoxUniform
 
 
 class UserNumpyUniform:
@@ -237,7 +238,7 @@ def test_prepare_sbi_problem(simulator: Callable, prior):
     simulator, prior = prepare_for_sbi(simulator, prior)
 
     # check batch sims and type
-    n_batch = 1
+    n_batch = 5
     assert simulator(prior.sample((n_batch,))).shape[0] == n_batch
     assert isinstance(simulator(prior.sample((1,))), Tensor)
     assert prior.sample().dtype == torch.float32
@@ -435,3 +436,41 @@ def test_passing_custom_density_estimator(arg):
         density_estimator = arg
     prior = MultivariateNormal(torch.zeros(2), torch.eye(2))
     _ = SNPE_C(prior=prior, density_estimator=density_estimator)
+
+
+def test_validate_theta_and_x_cpu():
+
+    cpu_device = torch.device('cpu')
+
+    theta = torch.ones((32,8), dtype=torch.float32).to(cpu_device)
+    x = torch.zeros((32,100), dtype=torch.float32).to(cpu_device)
+    plain_ft = torch.FloatTensor((32,8)) #using an explicit type
+
+    assert isinstance(theta, torch.Tensor), "cpu based torch.tensor is not an instance of torch.Tensor"
+    assert theta.dtype == torch.float32, f"cpu based torch.tensor(dtype=torch.float32) yields unexpected dtype {theta.dtype}."
+    assert isinstance(theta, torch.FloatTensor), "cpu based torch.tensor(dtype=torch.float32) is no FloatTensor."
+    assert plain_ft.dtype == torch.float32, "FloatTensor does not expose float32 dtype."
+
+    # test on cpu
+    validate_theta_and_x(theta, x)
+
+    with pytest.raises(AssertionError) as exc:
+        validate_theta_and_x(theta, x.to(torch.float64))
+
+
+@pytest.mark.gpu
+def test_validate_theta_and_x_gpu():
+
+    gpu_if_present = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
+
+    theta = torch.ones((32,8), dtype=torch.float32).to(gpu_if_present)
+    x = torch.zeros((32,100), dtype=torch.float32).to(gpu_if_present)
+
+    assert isinstance(theta, torch.Tensor), "gpu based torch.tensor is not an instance of torch.Tensor"
+    assert theta.dtype == torch.float32, f"gpu based torch.tensor(dtype=torch.float32) yields unexpected dtype {theta.dtype}."
+    assert not isinstance(theta, torch.FloatTensor), "gpu based torch.tensor(dtype=torch.float32) is FloatTensor, even though it shouldn't be."
+
+    validate_theta_and_x(theta, x)
+
+    with pytest.raises(AssertionError) as exc:
+        validate_theta_and_x(theta, x.to(torch.float64))
